==> invenio-shibboleth-0.1.1.tar.gz: invenio-shibboleth-0.1.1/invenio_shibboleth/handlers.py <==
# -*- coding: utf-8 -*-
#
# Copyright (C) 2019, 2020 Esteban J. Garcia Gabancho.
# Copyright (C) 2020 Mojib Wali.
#
# invenio-shibboleth is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
"""Default handlers for SSO-SAML."""

from flask import abort, current_app, session
from flask_login import current_user
from flask_security import logout_user
from invenio_db import db

from .invenio_accounts.errors import AlreadyLinkedError
from .invenio_accounts.utils import account_authenticate, account_get_user, \
    account_link_external_id, account_register, \
    create_csrf_disabled_registrationform, fill_form
from .invenio_app import get_safe_redirect_target


def account_info(attributes, remote_app):
    """Return account info for remote user.

    :param attributes: (dict) dictionary of data returned by identity provider.

    :param remote_app: (str) Identity provider key.

    :returns: (dict) A dictionary representing user to create or update.

    :mappings extracts the mapping or attributes for given remote_app.

    """
    if current_app.config['SSO_SAML_IDPS']:
        mappings = current_app.config['SSO_SAML_IDPS'][remote_app]['mappings']
    else:
        mappings = {
            "email": "email",
            "name": "name",
            "surname": "surname",
            "external_id": "external_id",
        }

    name = attributes[mappings['name']][0]
    surname = attributes[mappings['surname']][0]
    email = attributes[mappings['email']][0]
    external_id = attributes[mappings['external_id']][0]

    # custom attr
    if 'org_id' in attributes and 'org_id' in mappings:
        org_id = attributes[mappings['org_id']][0]
        # set the value to the session
        session['org_id'] = org_id

    if 'org_name' in attributes and 'org_name' in mappings:
        org_name = attributes[mappings['org_name']][0]
        # set the value to the session
        session['org_name'] = org_name

    if 'identifier' in attributes and 'identifier' in mappings:
        identifier = attributes[mappings['identifier']][0]
        # set the value to the session
        session['identifier'] = identifier

    return dict(
        user=dict(
            email=email,
            profile=dict(username=surname, full_name=name+" "+surname),
        ),
        external_id=external_id,
        external_method=remote_app,
        active=True
    )


def default_account_setup(user, account_info):
    """Default account setup which only links ``User`` and ``UserIdentity``."""
    try:
        account_link_external_id(
            user,
            dict(
                id=account_info['external_id'],
                method=account_info['external_method']))
    except AlreadyLinkedError:
        pass


def default_sls_handler(auth, next_url):
    """Default SLS handler which simply logs out the user."""
    logout_user()
    next_url = (get_safe_redirect_target(_target=next_url) or
                current_app.config['SECURITY_POST_LOGOUT_VIEW'])
    return next_url


def acs_handler_factory(remote_app, account_setup=default_account_setup):
    """Generate ACS handlers with an specific account info and setup functions.

    .. note::

        In 90% of the cases the ACS handler is going to be the same, only the
        way the information is extracted and processed from the IdP will be
        different.


    :param account_info: callable to extract the account information from a
        dict like object. This function is expected to return a dictionary
        similar to this:

        .. code-block:: python

            dict(
                user=dict(
                    email='federico@example.com',
                    profile=dict(username='federico',
                                 full_name='Federico Fernandez'),
                ),
                external_id='12345679abcdf',
                external_method='example',
                active=True
             )

        Where ``external_id`` is the ID provided by the IdP and
        ``external_method`` is the name of the IdP as in the configuration
        file (not mandatory but recommended).

    :param account_setup: callable to setup the user account with the
        corresponding IdP account information. Typically this means creating a
        new row under ``UserIdentity`` and maybe extending  ``g.identity``.

    :return: function to be used as ACS handler
    """
    def default_acs_handler(auth, next_url):
        """Default ACS handler.

        :para auth: A :class:`flask_sso_saml.utils.SAMLAuth` instance.
        :param next_url: String with the next URL to redirect to.

        :return: Next URL
        """
        if not current_user.is_authenticated:
            current_app.logger.debug(
                'Metadata received from IdP %s', auth.get_attributes()
            )
            _account_info = account_info(auth.get_attributes(), remote_app)
            current_app.logger.debug(
                'Metadata extracted from IdP %s', _account_info
            )
            # TODO: signals?

            user = account_get_user(_account_info)

            if user is None:
                form = create_csrf_disabled_registrationform()
                form = fill_form(form, _account_info['user'])
                user = account_register(form)

            # if registration fails ... TODO: signup?
            if user is None or not account_authenticate(user):
                abort(401)

            account_setup(user, _account_info)

        db.session.commit()

        next_url = (get_safe_redirect_target(_target=next_url) or
                    current_app.config['SECURITY_POST_LOGIN_VIEW'])
        return next_url

    return default_acs_handler

==> invenio-shibboleth-1.0.4.tar.gz: invenio-shibboleth-1.0.4/invenio_shibboleth/handlers.py <==
# -*- coding: utf-8 -*-
#
# Copyright (C) 2019, 2020 Esteban J. Garcia Gabancho.
# Copyright (C) 2020 Mojib Wali.
#
# invenio-shibboleth is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
"""Default handlers for SSO-SAML."""

from flask import abort, current_app, session
from flask_login import current_user
from flask_security import logout_user
from invenio_db import db

from .invenio_accounts.errors import AlreadyLinkedError
from .invenio_accounts.utils import (
    account_authenticate,
    account_get_user,
    account_link_external_id,
    account_register,
    create_csrf_disabled_registrationform,
    fill_form,
)
from .invenio_app import get_safe_redirect_target


def account_info(attributes, remote_app):
    """Return account info for remote user.

    :param attributes: (dict) dictionary of data returned by identity provider.

    :param remote_app: (str) Identity provider key.

    :returns: (dict) A dictionary representing user to create or update.

    :mappings extracts the mapping or attributes for given remote_app.

    """
    if current_app.config["SSO_SAML_IDPS"]:
        mappings = current_app.config["SSO_SAML_IDPS"][remote_app]["mappings"]
    else:
        mappings = {
            "email": "email",
            "name": "name",
            "surname": "surname",
            "external_id": "external_id",
        }

    name = attributes[mappings["name"]][0]
    surname = attributes[mappings["surname"]][0]
    email = attributes[mappings["email"]][0]
    external_id = attributes[mappings["external_id"]][0]

    # custom attr
    if "org_id" in attributes and "org_id" in mappings:
        org_id = attributes[mappings["org_id"]][0]
        # set the value to the session
        session["org_id"] = org_id

    if "org_name" in attributes and "org_name" in mappings:
        org_name = attributes[mappings["org_name"]][0]
        # set the value to the session
        session["org_name"] = org_name

    if "identifier" in attributes and "identifier" in mappings:
        identifier = attributes[mappings["identifier"]][0]
        # set the value to the session
        session["identifier"] = identifier

    # TODO: get prefix
    prefix = "TUG"

    return dict(
        user=dict(
            email=email,
            profile=dict(username=prefix + external_id.split('@')[0],
                         full_name=name + " " + surname),
        ),
        external_id=external_id,
        external_method=remote_app,
        active=True,
    )


def default_account_setup(user, account_info):
    """Default account setup which only links ``User`` and ``UserIdentity``."""
    try:
        account_link_external_id(
            user,
            dict(
                id=account_info["external_id"], method=account_info["external_method"]
            ),
        )
    except AlreadyLinkedError:
        pass


def default_sls_handler(auth, next_url):
    """Default SLS handler which simply logs out the user."""
    logout_user()
    next_url = (
        get_safe_redirect_target(_target=next_url) or current_app.config["SECURITY_POST_LOGOUT_VIEW"]
    )
    return next_url


def acs_handler_factory(remote_app, account_setup=default_account_setup):
    """Generate ACS handlers with an specific account info and setup functions.

    .. note::

        In 90% of the cases the ACS handler is going to be the same, only the
        way the information is extracted and processed from the IdP will be
        different.


    :param account_info: callable to extract the account information from a
        dict like object. This function is expected to return a dictionary
        similar to this:

        .. code-block:: python

            dict(
                user=dict(
                    email='federico@example.com',
                    profile=dict(username='federico',
                                 full_name='Federico Fernandez'),
                ),
                external_id='12345679abcdf',
                external_method='example',
                active=True
             )

        Where ``external_id`` is the ID provided by the IdP and
        ``external_method`` is the name of the IdP as in the configuration
        file (not mandatory but recommended).

    :param account_setup: callable to setup the user account with the
        corresponding IdP account information. Typically this means creating a
        new row under ``UserIdentity`` and maybe extending  ``g.identity``.

    :return: function to be used as ACS handler
    """

    def default_acs_handler(auth, next_url):
        """Default ACS handler.

        :para auth: A :class:`flask_sso_saml.utils.SAMLAuth` instance.
        :param next_url: String with the next URL to redirect to.

        :return: Next URL
        """
        if not current_user.is_authenticated:
            current_app.logger.debug(
                "Metadata received from IdP %s", auth.get_attributes()
            )
            _account_info = account_info(auth.get_attributes(), remote_app)
            current_app.logger.debug("Metadata extracted from IdP %s", _account_info)
            # TODO: signals?

            user = account_get_user(_account_info)

            if user is None:
                form = create_csrf_disabled_registrationform()
                form = fill_form(form, _account_info["user"])
                user = account_register(form)

            # if registration fails ... TODO: signup?
            if user is None or not account_authenticate(user):
                abort(401)

            account_setup(user, _account_info)

        db.session.commit()

        next_url = (
            get_safe_redirect_target(_target=next_url) or current_app.config["SECURITY_POST_LOGIN_VIEW"]
        )
        return next_url

    return default_acs_handler
